@@ -2,7 +2,7 @@
 
 from __future__ import annotations
 
-from typing import Protocol, TYPE_CHECKING
+from typing import TYPE_CHECKING, Protocol
 
 if TYPE_CHECKING:
     from crewai.rag.chromadb.client import ChromaDBClient
