@@ -1,6 +1,9 @@
 from abc import ABC, abstractmethod
 from typing import Any
 
+from crewai.rag.core.base_embeddings_provider import BaseEmbeddingsProvider
+from crewai.rag.embeddings.types import ProviderSpec
+
 
 class BaseRAGStorage(ABC):
     """
@@ -13,7 +16,7 @@ class BaseRAGStorage(ABC):
         self,
         type: str,
         allow_reset: bool = True,
-        embedder_config: dict[str, Any] | None = None,
+        embedder_config: ProviderSpec | BaseEmbeddingsProvider | None = None,
         crew: Any = None,
     ):
         self.type = type
