@@ -1,5 +1,3 @@
-from typing import Dict, Optional, Union
-
 from pydantic import BaseModel, Field
 
 from crewai.tools.base_tool import BaseTool
@@ -10,7 +8,7 @@ i18n = I18N()
 
 class AddImageToolSchema(BaseModel):
     image_url: str = Field(..., description="The URL or path of the image to add")
-    action: Optional[str] = Field(
+    action: str | None = Field(
         default=None, description="Optional context or question about the image"
     )
 
@@ -18,14 +16,16 @@ class AddImageToolSchema(BaseModel):
 class AddImageTool(BaseTool):
     """Tool for adding images to the content"""
 
-    name: str = Field(default_factory=lambda: i18n.tools("add_image")["name"])  # type: ignore
-    description: str = Field(default_factory=lambda: i18n.tools("add_image")["description"])  # type: ignore
+    name: str = Field(default_factory=lambda: i18n.tools("add_image")["name"])  # type: ignore[index]
+    description: str = Field(
+        default_factory=lambda: i18n.tools("add_image")["description"]  # type: ignore[index]
+    )
     args_schema: type[BaseModel] = AddImageToolSchema
 
     def _run(
         self,
         image_url: str,
-        action: Optional[str] = None,
+        action: str | None = None,
         **kwargs,
     ) -> dict:
         action = action or i18n.tools("add_image")["default_action"]  # type: ignore
