@@ -9,9 +9,9 @@ from .delegate_work_tool import DelegateWorkTool
 class AgentTools:
     """Manager class for agent-related tools"""
 
-    def __init__(self, agents: list[BaseAgent], i18n: I18N = I18N()):
+    def __init__(self, agents: list[BaseAgent], i18n: I18N | None = None):
         self.agents = agents
-        self.i18n = i18n
+        self.i18n = i18n if i18n is not None else I18N()
 
     def tools(self) -> list[BaseTool]:
         """Get all available agent tools"""
