@@ -1,4 +1,4 @@
-from typing import Any, Dict, Optional
+from typing import Any
 
 from pydantic import BaseModel, Field
 from pydantic import BaseModel as PydanticBaseModel
@@ -7,7 +7,7 @@ from pydantic import Field as PydanticField
 
 class ToolCalling(BaseModel):
     tool_name: str = Field(..., description="The name of the tool to be called.")
-    arguments: Optional[Dict[str, Any]] = Field(
+    arguments: dict[str, Any] | None = Field(
         ..., description="A dictionary of arguments to be passed to the tool."
     )
 
@@ -16,6 +16,6 @@ class InstructorToolCalling(PydanticBaseModel):
     tool_name: str = PydanticField(
         ..., description="The name of the tool to be called."
     )
-    arguments: Optional[Dict[str, Any]] = PydanticField(
+    arguments: dict[str, Any] | None = PydanticField(
         ..., description="A dictionary of arguments to be passed to the tool."
     )
