@@ -1,62 +1,55 @@
+import logging
 import os
-from typing import Any, Dict, List, Optional, Union
+from typing import Any, Final
 
 from crewai.cli.constants import DEFAULT_LLM_MODEL, ENV_VARS, LITELLM_PARAMS
-from crewai.llm import LLM, BaseLLM
+from crewai.llm import LLM
+from crewai.llms.base_llm import BaseLLM
+
+logger = logging.getLogger(__name__)
 
 
 def create_llm(
-    llm_value: Union[str, LLM, Any, None] = None,
-) -> Optional[LLM | BaseLLM]:
-    """
-    Creates or returns an LLM instance based on the given llm_value.
+    llm_value: str | LLM | Any | None = None,
+) -> LLM | BaseLLM | None:
+    """Creates or returns an LLM instance based on the given llm_value.
 
     Args:
-        llm_value (str | BaseLLM | Any | None):
-            - str: The model name (e.g., "gpt-4").
-            - BaseLLM: Already instantiated BaseLLM (including LLM), returned as-is.
-            - Any: Attempt to extract known attributes like model_name, temperature, etc.
-            - None: Use environment-based or fallback default model.
+        llm_value: LLM instance, model name string, None, or an object with LLM attributes.
 
     Returns:
         A BaseLLM instance if successful, or None if something fails.
     """
 
-    # 1) If llm_value is already a BaseLLM or LLM object, return it directly
-    if isinstance(llm_value, LLM) or isinstance(llm_value, BaseLLM):
+    if isinstance(llm_value, (LLM, BaseLLM)):
         return llm_value
 
-    # 2) If llm_value is a string (model name)
     if isinstance(llm_value, str):
         try:
-            created_llm = LLM(model=llm_value)
-            return created_llm
+            return LLM(model=llm_value)
         except Exception as e:
-            print(f"Failed to instantiate LLM with model='{llm_value}': {e}")
+            logger.debug(f"Failed to instantiate LLM with model='{llm_value}': {e}")
             return None
 
-    # 3) If llm_value is None, parse environment variables or use default
     if llm_value is None:
         return _llm_via_environment_or_fallback()
 
-    # 4) Otherwise, attempt to extract relevant attributes from an unknown object
     try:
-        # Extract attributes with explicit types
         model = (
             getattr(llm_value, "model", None)
             or getattr(llm_value, "model_name", None)
             or getattr(llm_value, "deployment_name", None)
             or str(llm_value)
         )
-        temperature: Optional[float] = getattr(llm_value, "temperature", None)
-        max_tokens: Optional[int] = getattr(llm_value, "max_tokens", None)
-        logprobs: Optional[int] = getattr(llm_value, "logprobs", None)
-        timeout: Optional[float] = getattr(llm_value, "timeout", None)
-        api_key: Optional[str] = getattr(llm_value, "api_key", None)
-        base_url: Optional[str] = getattr(llm_value, "base_url", None)
-        api_base: Optional[str] = getattr(llm_value, "api_base", None)
+        temperature: float | None = getattr(llm_value, "temperature", None)
+        max_tokens: int | None = getattr(llm_value, "max_tokens", None)
+        logprobs: int | None = getattr(llm_value, "logprobs", None)
+        timeout: float | None = getattr(llm_value, "timeout", None)
+        api_key: str | None = getattr(llm_value, "api_key", None)
+        base_url: str | None = getattr(llm_value, "base_url", None)
+        api_base: str | None = getattr(llm_value, "api_base", None)
 
-        created_llm = LLM(
+        return LLM(
             model=model,
             temperature=temperature,
             max_tokens=max_tokens,
@@ -66,15 +59,23 @@ def create_llm(
             base_url=base_url,
             api_base=api_base,
         )
-        return created_llm
     except Exception as e:
-        print(f"Error instantiating LLM from unknown object type: {e}")
+        logger.debug(f"Error instantiating LLM from unknown object type: {e}")
         return None
 
 
-def _llm_via_environment_or_fallback() -> Optional[LLM]:
-    """
-    Helper function: if llm_value is None, we load environment variables or fallback default model.
+UNACCEPTED_ATTRIBUTES: Final[list[str]] = [
+    "AWS_ACCESS_KEY_ID",
+    "AWS_SECRET_ACCESS_KEY",
+    "AWS_REGION_NAME",
+]
+
+
+def _llm_via_environment_or_fallback() -> LLM | None:
+    """Creates an LLM instance based on environment variables or defaults.
+
+    Returns:
+        A BaseLLM instance if successful, or None if something fails.
     """
     model_name = (
         os.environ.get("MODEL")
@@ -83,28 +84,25 @@ def _llm_via_environment_or_fallback() -> Optional[LLM]:
         or DEFAULT_LLM_MODEL
     )
 
-    # Initialize parameters with correct types
     model: str = model_name
-    temperature: Optional[float] = None
-    max_tokens: Optional[int] = None
-    max_completion_tokens: Optional[int] = None
-    logprobs: Optional[int] = None
-    timeout: Optional[float] = None
-    api_key: Optional[str] = None
-    base_url: Optional[str] = None
-    api_version: Optional[str] = None
-    presence_penalty: Optional[float] = None
-    frequency_penalty: Optional[float] = None
-    top_p: Optional[float] = None
-    n: Optional[int] = None
-    stop: Optional[Union[str, List[str]]] = None
-    logit_bias: Optional[Dict[int, float]] = None
-    response_format: Optional[Dict[str, Any]] = None
-    seed: Optional[int] = None
-    top_logprobs: Optional[int] = None
-    callbacks: List[Any] = []
+    temperature: float | None = None
+    max_tokens: int | None = None
+    max_completion_tokens: int | None = None
+    logprobs: int | None = None
+    timeout: float | None = None
+    api_key: str | None = None
+    api_version: str | None = None
+    presence_penalty: float | None = None
+    frequency_penalty: float | None = None
+    top_p: float | None = None
+    n: int | None = None
+    stop: str | list[str] | None = None
+    logit_bias: dict[int, float] | None = None
+    response_format: dict[str, Any] | None = None
+    seed: int | None = None
+    top_logprobs: int | None = None
+    callbacks: list[Any] = []
 
-    # Optional base URL from env
     base_url = (
         os.environ.get("BASE_URL")
         or os.environ.get("OPENAI_API_BASE")
@@ -119,8 +117,7 @@ def _llm_via_environment_or_fallback() -> Optional[LLM]:
     elif api_base and not base_url:
         base_url = api_base
 
-    # Initialize llm_params dictionary
-    llm_params: Dict[str, Any] = {
+    llm_params: dict[str, Any] = {
         "model": model,
         "temperature": temperature,
         "max_tokens": max_tokens,
@@ -143,11 +140,6 @@ def _llm_via_environment_or_fallback() -> Optional[LLM]:
         "callbacks": callbacks,
     }
 
-    UNACCEPTED_ATTRIBUTES = [
-        "AWS_ACCESS_KEY_ID",
-        "AWS_SECRET_ACCESS_KEY",
-        "AWS_REGION_NAME",
-    ]
     set_provider = model_name.partition("/")[0] if "/" in model_name else "openai"
 
     if set_provider in ENV_VARS:
@@ -167,28 +159,26 @@ def _llm_via_environment_or_fallback() -> Optional[LLM]:
                             if key not in ["prompt", "key_name", "default"]:
                                 llm_params[key.lower()] = value
                 else:
-                    print(
+                    logger.debug(
                         f"Expected env_var to be a dictionary, but got {type(env_var)}"
                     )
 
-    # Remove None values
     llm_params = {k: v for k, v in llm_params.items() if v is not None}
 
-    # Try creating the LLM
     try:
-        new_llm = LLM(**llm_params)
-        return new_llm
+        return LLM(**llm_params)
     except Exception as e:
-        print(
+        logger.debug(
             f"Error instantiating LLM from environment/fallback: {type(e).__name__}: {e}"
         )
         return None
 
 
 def _normalize_key_name(key_name: str) -> str:
-    """
-    Maps environment variable names to recognized litellm parameter keys,
-    using patterns from LITELLM_PARAMS.
+    """Maps environment variable names to recognized litellm parameter keys.
+
+    Args:
+        key_name: The environment variable name to normalize.
     """
     for pattern in LITELLM_PARAMS:
         if pattern in key_name:
