@@ -1,8 +1,7 @@
 from unittest.mock import MagicMock, patch
 
 import pytest
-from mem0.client.main import MemoryClient
-from mem0.memory.main import Memory
+from mem0 import Memory, MemoryClient
 
 from crewai.memory.storage.mem0_storage import Mem0Storage
 
@@ -13,11 +12,71 @@ class MockCrew:
         self.agents = [MagicMock(role="Test Agent")]
 
 
+# Test data constants
+SYSTEM_CONTENT = (
+    "You are Friendly chatbot assistant. You are a kind and "
+    "knowledgeable chatbot assistant. You excel at understanding user needs, "
+    "providing helpful responses, and maintaining engaging conversations. "
+    "You remember previous interactions to provide a personalized experience.\n"
+    "Your personal goal is: Engage in useful and interesting conversations "
+    "with users while remembering context.\n"
+    "To give my best complete final answer to the task respond using the exact "
+    "following format:\n\n"
+    "Thought: I now can give a great answer\n"
+    "Final Answer: Your final answer must be the great and the most complete "
+    "as possible, it must be outcome described.\n\n"
+    "I MUST use these formats, my job depends on it!"
+)
+
+USER_CONTENT = (
+    "\nCurrent Task: Respond to user conversation. User message: "
+    "What do you know about me?\n\n"
+    "This is the expected criteria for your final answer: Contextually "
+    "appropriate, helpful, and friendly response.\n"
+    "you MUST return the actual complete content as the final answer, "
+    "not a summary.\n\n"
+    "# Useful context: \nExternal memories:\n"
+    "- User is from India\n"
+    "- User is interested in the solar system\n"
+    "- User name is Vidit Ostwal\n"
+    "- User is interested in French cuisine\n\n"
+    "Begin! This is VERY important to you, use the tools available and give "
+    "your best Final Answer, your job depends on it!\n\n"
+    "Thought:"
+)
+
+ASSISTANT_CONTENT = (
+    "I now can give a great answer  \n"
+    "Final Answer: Hi Vidit! From our previous conversations, I know you're "
+    "from India and have a great interest in the solar system. It's fascinating "
+    "to explore the wonders of space, isn't it? Also, I remember you have a "
+    "passion for French cuisine, which has so many delightful dishes to explore. "
+    "If there's anything specific you'd like to discuss or learn about—whether "
+    "it's about the solar system or some great French recipes—feel free to let "
+    "me know! I'm here to help."
+)
+
+TEST_DESCRIPTION = (
+    "Respond to user conversation. User message: What do you know about me?"
+)
+
+# Extracted content (after processing by _get_user_message and _get_assistant_message)
+EXTRACTED_USER_CONTENT = "What do you know about me?"
+EXTRACTED_ASSISTANT_CONTENT = (
+    "Hi Vidit! From our previous conversations, I know you're "
+    "from India and have a great interest in the solar system. It's fascinating "
+    "to explore the wonders of space, isn't it? Also, I remember you have a "
+    "passion for French cuisine, which has so many delightful dishes to explore. "
+    "If there's anything specific you'd like to discuss or learn about—whether "
+    "it's about the solar system or some great French recipes—feel free to let "
+    "me know! I'm here to help."
+)
+
+
 @pytest.fixture
 def mock_mem0_memory():
     """Fixture to create a mock Memory instance"""
-    mock_memory = MagicMock(spec=Memory)
-    return mock_memory
+    return MagicMock(spec=Memory)
 
 
 @pytest.fixture
@@ -25,7 +84,9 @@ def mem0_storage_with_mocked_config(mock_mem0_memory):
     """Fixture to create a Mem0Storage instance with mocked dependencies"""
 
     # Patch the Memory class to return our mock
-    with patch("mem0.memory.main.Memory.from_config", return_value=mock_mem0_memory) as mock_from_config:
+    with patch(
+        "mem0.Memory.from_config", return_value=mock_mem0_memory
+    ) as mock_from_config:
         config = {
             "vector_store": {
                 "provider": "mock_vector_store",
@@ -56,7 +117,14 @@ def mem0_storage_with_mocked_config(mock_mem0_memory):
         # Parameters like run_id, includes, and excludes doesn't matter in Memory OSS
         crew = MockCrew()
 
-        embedder_config={"user_id": "test_user", "local_mem0_config": config, "run_id": "my_run_id", "includes": "include1","excludes": "exclude1", "infer" : True}
+        embedder_config = {
+            "user_id": "test_user",
+            "local_mem0_config": config,
+            "run_id": "my_run_id",
+            "includes": "include1",
+            "excludes": "exclude1",
+            "infer": True,
+        }
 
         mem0_storage = Mem0Storage(type="short_term", crew=crew, config=embedder_config)
         return mem0_storage, mock_from_config, config
@@ -73,8 +141,7 @@ def test_mem0_storage_initialization(mem0_storage_with_mocked_config, mock_mem0_
 @pytest.fixture
 def mock_mem0_memory_client():
     """Fixture to create a mock MemoryClient instance"""
-    mock_memory = MagicMock(spec=MemoryClient)
-    return mock_memory
+    return MagicMock(spec=MemoryClient)
 
 
 @pytest.fixture
@@ -85,34 +152,35 @@ def mem0_storage_with_memory_client_using_config_from_crew(mock_mem0_memory_clie
     with patch.object(MemoryClient, "__new__", return_value=mock_mem0_memory_client):
         crew = MockCrew()
 
-        embedder_config={
-                    "user_id": "test_user",
-                    "api_key": "ABCDEFGH",
-                    "org_id": "my_org_id",
-                    "project_id": "my_project_id",
-                    "run_id": "my_run_id",
-                    "includes": "include1",
-                    "excludes": "exclude1",
-                    "infer": True
-                }
+        embedder_config = {
+            "user_id": "test_user",
+            "api_key": "ABCDEFGH",
+            "org_id": "my_org_id",
+            "project_id": "my_project_id",
+            "run_id": "my_run_id",
+            "includes": "include1",
+            "excludes": "exclude1",
+            "infer": True,
+        }
 
-        mem0_storage = Mem0Storage(type="short_term", crew=crew, config=embedder_config)
-        return mem0_storage
+        return Mem0Storage(type="short_term", crew=crew, config=embedder_config)
 
 
 @pytest.fixture
-def mem0_storage_with_memory_client_using_explictly_config(mock_mem0_memory_client, mock_mem0_memory):
+def mem0_storage_with_memory_client_using_explictly_config(
+    mock_mem0_memory_client, mock_mem0_memory
+):
     """Fixture to create a Mem0Storage instance with mocked dependencies"""
 
     # We need to patch both MemoryClient and Memory to prevent actual initialization
-    with patch.object(MemoryClient, "__new__", return_value=mock_mem0_memory_client), \
-         patch.object(Memory, "__new__", return_value=mock_mem0_memory):
-
+    with (
+        patch.object(MemoryClient, "__new__", return_value=mock_mem0_memory_client),
+        patch.object(Memory, "__new__", return_value=mock_mem0_memory),
+    ):
         crew = MockCrew()
         new_config = {"provider": "mem0", "config": {"api_key": "new-api-key"}}
 
-        mem0_storage = Mem0Storage(type="short_term", crew=crew, config=new_config)
-        return mem0_storage
+        return Mem0Storage(type="short_term", crew=crew, config=new_config)
 
 
 def test_mem0_storage_with_memory_client_initialization(
@@ -142,18 +210,23 @@ def test_mem0_storage_updates_project_with_custom_categories(mock_mem0_memory_cl
     mock_mem0_memory_client.update_project = MagicMock()
 
     new_categories = [
-    {"lifestyle_management_concerns": "Tracks daily routines, habits, hobbies and interests including cooking, time management and work-life balance"},
+        {
+            "lifestyle_management_concerns": (
+                "Tracks daily routines, habits, hobbies and interests "
+                "including cooking, time management and work-life balance"
+            )
+        },
     ]
 
     crew = MockCrew()
 
-    config={
-            "user_id": "test_user",
-            "api_key": "ABCDEFGH",
-            "org_id": "my_org_id",
-            "project_id": "my_project_id",
-            "custom_categories": new_categories
-        }
+    config = {
+        "user_id": "test_user",
+        "api_key": "ABCDEFGH",
+        "org_id": "my_org_id",
+        "project_id": "my_project_id",
+        "custom_categories": new_categories,
+    }
 
     with patch.object(MemoryClient, "__new__", return_value=mock_mem0_memory_client):
         _ = Mem0Storage(type="short_term", crew=crew, config=config)
@@ -163,8 +236,6 @@ def test_mem0_storage_updates_project_with_custom_categories(mock_mem0_memory_cl
     )
 
 
-
-
 def test_save_method_with_memory_oss(mem0_storage_with_mocked_config):
     """Test save method for different memory types"""
     mem0_storage, _, _ = mem0_storage_with_mocked_config
@@ -172,68 +243,134 @@ def test_save_method_with_memory_oss(mem0_storage_with_mocked_config):
 
     # Test short_term memory type (already set in fixture)
     test_value = "This is a test memory"
-    test_metadata = {"key": "value"}
+    test_metadata = {
+        "description": TEST_DESCRIPTION,
+        "messages": [
+            {"role": "system", "content": SYSTEM_CONTENT},
+            {"role": "user", "content": USER_CONTENT},
+            {"role": "assistant", "content": ASSISTANT_CONTENT},
+        ],
+        "agent": "Friendly chatbot assistant",
+    }
 
     mem0_storage.save(test_value, test_metadata)
 
     mem0_storage.memory.add.assert_called_once_with(
-        [{"role": "assistant" , "content": test_value}],
+        [
+            {"role": "user", "content": EXTRACTED_USER_CONTENT},
+            {
+                "role": "assistant",
+                "content": EXTRACTED_ASSISTANT_CONTENT,
+            },
+        ],
         infer=True,
-        metadata={"type": "short_term", "key": "value"},
+        metadata={
+            "type": "short_term",
+            "description": TEST_DESCRIPTION,
+            "agent": "Friendly chatbot assistant",
+        },
         run_id="my_run_id",
         user_id="test_user",
-        agent_id='Test_Agent'
+        agent_id="Test_Agent",
     )
 
+
 def test_save_method_with_multiple_agents(mem0_storage_with_mocked_config):
     mem0_storage, _, _ = mem0_storage_with_mocked_config
-    mem0_storage.crew.agents = [MagicMock(role="Test Agent"), MagicMock(role="Test Agent 2"), MagicMock(role="Test Agent 3")]
+    mem0_storage.crew.agents = [
+        MagicMock(role="Test Agent"),
+        MagicMock(role="Test Agent 2"),
+        MagicMock(role="Test Agent 3"),
+    ]
     mem0_storage.memory.add = MagicMock()
 
     test_value = "This is a test memory"
-    test_metadata = {"key": "value"}
+    test_metadata = {
+        "description": TEST_DESCRIPTION,
+        "messages": [
+            {"role": "system", "content": SYSTEM_CONTENT},
+            {"role": "user", "content": USER_CONTENT},
+            {"role": "assistant", "content": ASSISTANT_CONTENT},
+        ],
+        "agent": "Friendly chatbot assistant",
+    }
 
     mem0_storage.save(test_value, test_metadata)
 
     mem0_storage.memory.add.assert_called_once_with(
-        [{"role": "assistant" , "content": test_value}],
+        [
+            {"role": "user", "content": EXTRACTED_USER_CONTENT},
+            {
+                "role": "assistant",
+                "content": EXTRACTED_ASSISTANT_CONTENT,
+            },
+        ],
         infer=True,
-        metadata={"type": "short_term", "key": "value"},
+        metadata={
+            "type": "short_term",
+            "description": TEST_DESCRIPTION,
+            "agent": "Friendly chatbot assistant",
+        },
         run_id="my_run_id",
         user_id="test_user",
-        agent_id='Test_Agent_Test_Agent_2_Test_Agent_3'
+        agent_id="Test_Agent_Test_Agent_2_Test_Agent_3",
     )
 
 
-def test_save_method_with_memory_client(mem0_storage_with_memory_client_using_config_from_crew):
+def test_save_method_with_memory_client(
+    mem0_storage_with_memory_client_using_config_from_crew,
+):
     """Test save method for different memory types"""
     mem0_storage = mem0_storage_with_memory_client_using_config_from_crew
     mem0_storage.memory.add = MagicMock()
 
     # Test short_term memory type (already set in fixture)
     test_value = "This is a test memory"
-    test_metadata = {"key": "value"}
+    test_metadata = {
+        "description": TEST_DESCRIPTION,
+        "messages": [
+            {"role": "system", "content": SYSTEM_CONTENT},
+            {"role": "user", "content": USER_CONTENT},
+            {"role": "assistant", "content": ASSISTANT_CONTENT},
+        ],
+        "agent": "Friendly chatbot assistant",
+    }
 
     mem0_storage.save(test_value, test_metadata)
 
     mem0_storage.memory.add.assert_called_once_with(
-        [{'role': 'assistant' , 'content': test_value}],
+        [
+            {"role": "user", "content": EXTRACTED_USER_CONTENT},
+            {
+                "role": "assistant",
+                "content": EXTRACTED_ASSISTANT_CONTENT,
+            },
+        ],
         infer=True,
-        metadata={"type": "short_term", "key": "value"},
+        metadata={
+            "type": "short_term",
+            "description": TEST_DESCRIPTION,
+            "agent": "Friendly chatbot assistant",
+        },
         version="v2",
         run_id="my_run_id",
         includes="include1",
         excludes="exclude1",
-        output_format='v1.1',
-        user_id='test_user',
-         agent_id='Test_Agent'
+        output_format="v1.1",
+        user_id="test_user",
+        agent_id="Test_Agent",
     )
 
 
 def test_search_method_with_memory_oss(mem0_storage_with_mocked_config):
     """Test search method for different memory types"""
     mem0_storage, _, _ = mem0_storage_with_mocked_config
-    mock_results = {"results": [{"score": 0.9, "memory": "Result 1"}, {"score": 0.4, "memory": "Result 2"}]}
+    mock_results = {
+        "results": [
+            {"score": 0.9, "memory": "Result 1"},
+            {"score": 0.4, "memory": "Result 2"},
+        ]
+    }
     mem0_storage.memory.search = MagicMock(return_value=mock_results)
 
     results = mem0_storage.search("test query", limit=5, score_threshold=0.5)
@@ -242,18 +379,25 @@ def test_search_method_with_memory_oss(mem0_storage_with_mocked_config):
         query="test query",
         limit=5,
         user_id="test_user",
-        filters={'AND': [{'run_id': 'my_run_id'}]},
-        threshold=0.5
+        filters={"AND": [{"run_id": "my_run_id"}]},
+        threshold=0.5,
     )
 
     assert len(results) == 2
-    assert results[0]["context"] == "Result 1"
+    assert results[0]["content"] == "Result 1"
 
 
-def test_search_method_with_memory_client(mem0_storage_with_memory_client_using_config_from_crew):
+def test_search_method_with_memory_client(
+    mem0_storage_with_memory_client_using_config_from_crew,
+):
     """Test search method for different memory types"""
     mem0_storage = mem0_storage_with_memory_client_using_config_from_crew
-    mock_results = {"results": [{"score": 0.9, "memory": "Result 1"}, {"score": 0.4, "memory": "Result 2"}]}
+    mock_results = {
+        "results": [
+            {"score": 0.9, "memory": "Result 1"},
+            {"score": 0.4, "memory": "Result 2"},
+        ]
+    }
     mem0_storage.memory.search = MagicMock(return_value=mock_results)
 
     results = mem0_storage.search("test query", limit=5, score_threshold=0.5)
@@ -263,15 +407,15 @@ def test_search_method_with_memory_client(mem0_storage_with_memory_client_using_
         limit=5,
         metadata={"type": "short_term"},
         user_id="test_user",
-        version='v2',
+        version="v2",
         run_id="my_run_id",
-        output_format='v1.1',
-        filters={'AND': [{'run_id': 'my_run_id'}]},
-        threshold=0.5
+        output_format="v1.1",
+        filters={"AND": [{"run_id": "my_run_id"}]},
+        threshold=0.5,
     )
 
     assert len(results) == 2
-    assert results[0]["context"] == "Result 1"
+    assert results[0]["content"] == "Result 1"
 
 
 def test_mem0_storage_default_infer_value(mock_mem0_memory_client):
@@ -279,14 +423,12 @@ def test_mem0_storage_default_infer_value(mock_mem0_memory_client):
     with patch.object(MemoryClient, "__new__", return_value=mock_mem0_memory_client):
         crew = MockCrew()
 
-        config={
-                "user_id": "test_user",
-                "api_key": "ABCDEFGH"
-            }
+        config = {"user_id": "test_user", "api_key": "ABCDEFGH"}
 
         mem0_storage = Mem0Storage(type="short_term", crew=crew, config=config)
         assert mem0_storage.infer is True
 
+
 def test_save_memory_using_agent_entity(mock_mem0_memory_client):
     config = {
         "agent_id": "agent-123",
@@ -297,19 +439,25 @@ def test_save_memory_using_agent_entity(mock_mem0_memory_client):
         mem0_storage = Mem0Storage(type="external", config=config)
         mem0_storage.save("test memory", {"key": "value"})
         mem0_storage.memory.add.assert_called_once_with(
-            [{'role': 'assistant' , 'content': 'test memory'}],
+            [{"role": "assistant", "content": "test memory"}],
             infer=True,
             metadata={"type": "external", "key": "value"},
             agent_id="agent-123",
         )
 
+
 def test_search_method_with_agent_entity():
     config = {
         "agent_id": "agent-123",
     }
 
     mock_memory = MagicMock(spec=Memory)
-    mock_results = {"results": [{"score": 0.9, "memory": "Result 1"}, {"score": 0.4, "memory": "Result 2"}]}
+    mock_results = {
+        "results": [
+            {"score": 0.9, "memory": "Result 1"},
+            {"score": 0.4, "memory": "Result 2"},
+        ]
+    }
 
     with patch.object(Memory, "__new__", return_value=mock_memory):
         mem0_storage = Mem0Storage(type="external", config=config)
@@ -318,22 +466,29 @@ def test_search_method_with_agent_entity():
         results = mem0_storage.search("test query", limit=5, score_threshold=0.5)
 
         mem0_storage.memory.search.assert_called_once_with(
-        query="test query",
-        limit=5,
-        filters={"AND": [{"agent_id": "agent-123"}]},
-        threshold=0.5,
-    )
+            query="test query",
+            limit=5,
+            filters={"AND": [{"agent_id": "agent-123"}]},
+            threshold=0.5,
+        )
 
         assert len(results) == 2
-        assert results[0]["context"] == "Result 1"
+        assert results[0]["content"] == "Result 1"
 
 
 def test_search_method_with_agent_id_and_user_id():
     mock_memory = MagicMock(spec=Memory)
-    mock_results = {"results": [{"score": 0.9, "memory": "Result 1"}, {"score": 0.4, "memory": "Result 2"}]}
+    mock_results = {
+        "results": [
+            {"score": 0.9, "memory": "Result 1"},
+            {"score": 0.4, "memory": "Result 2"},
+        ]
+    }
 
     with patch.object(Memory, "__new__", return_value=mock_memory):
-        mem0_storage = Mem0Storage(type="external", config={"agent_id": "agent-123", "user_id": "user-123"})
+        mem0_storage = Mem0Storage(
+            type="external", config={"agent_id": "agent-123", "user_id": "user-123"}
+        )
 
         mem0_storage.memory.search = MagicMock(return_value=mock_results)
         results = mem0_storage.search("test query", limit=5, score_threshold=0.5)
@@ -341,10 +496,10 @@ def test_search_method_with_agent_id_and_user_id():
         mem0_storage.memory.search.assert_called_once_with(
             query="test query",
             limit=5,
-            user_id='user-123',
+            user_id="user-123",
             filters={"OR": [{"user_id": "user-123"}, {"agent_id": "agent-123"}]},
             threshold=0.5,
         )
 
         assert len(results) == 2
-        assert results[0]["context"] == "Result 1"
+        assert results[0]["content"] == "Result 1"
